import dataclasses
from typing import List

from pydantic import BaseModel

from hmmer_tables.csv_iter import csv_iter
from hmmer_tables.interval import PyInterval, RInterval
from hmmer_tables.path_like import PathLike

__all__ = [
    "DomTBLCoord",
    "DomTBLDomScore",
    "DomTBLIndex",
    "DomTBLRow",
    "DomTBLSeqScore",
    "read_domtbl",
]


class DomTBLIndex(BaseModel):
    name: str
    accession: str
    length: int


class DomTBLSeqScore(BaseModel):
    e_value: str
    score: str
    bias: str


class DomTBLDomScore(BaseModel):
    id: int
    size: int
    c_value: str
    i_value: str
    score: str
    bias: str


class DomTBLCoord(BaseModel):
    """
    Coordinates.

    Parameters
    ----------
    start
        Start coordinate, starting from 1. Consider using :method:`.interval` instead.
    stop
        Stop coordinate. `(start, stop)` form a closed interval. Consider using
        :method:`.interval` instead.
    """

    start: int
    stop: int

    @property
    def interval(self) -> PyInterval:
        """
        0-start, half-open interval.

        Returns
        -------
        PyInterval
            Interval.
        """
        rinterval = RInterval(self.start, self.stop)
        return rinterval.to_pyinterval()


class DomTBLRow(BaseModel):
    target: DomTBLIndex
    query: DomTBLIndex
    full_sequence: DomTBLSeqScore
    domain: DomTBLDomScore
    hmm_coord: DomTBLCoord
    ali_coord: DomTBLCoord
    env_coord: DomTBLCoord
    acc: str
    description: str

    def _asdict(self):
        return dataclasses.asdict(self)

    def __iter__(self):
        return iter(self._asdict().values())

    def _field_types(self):
        return {f.name: f.type for f in dataclasses.fields(self)}


def read_domtbl(filename: PathLike) -> List[DomTBLRow]:
    """
    Read domtbl file type.

    Parameters
    ----------
    file
        File path or file stream.
    """
    rows = []
    with open(filename, "r") as file:
        for x in csv_iter(file):
            row = DomTBLRow(
                target=DomTBLIndex(name=x[0], accession=x[1], length=int(x[2])),
                query=DomTBLIndex(name=x[3], accession=x[4], length=int(x[5])),
                full_sequence=DomTBLSeqScore(e_value=x[6], score=x[7], bias=x[8]),
                domain=DomTBLDomScore(
                    id=int(x[9]),
                    size=int(x[10]),
                    c_value=x[11],
                    i_value=x[12],
                    score=x[13],
                    bias=x[14],
                ),
                hmm_coord=DomTBLCoord(start=int(x[15]), stop=int(x[16])),
                ali_coord=DomTBLCoord(start=int(x[17]), stop=int(x[18])),
                env_coord=DomTBLCoord(start=int(x[19]), stop=int(x[20])),
                acc=x[21],
                description=" ".join(x[22:]),
            )
            rows.append(row)
    return rows



import csv
from io import TextIOWrapper

__all__ = ["csv_iter"]


def csv_iter(stream: TextIOWrapper):
    return csv.reader(uncomment(stream), delimiter=" ", skipinitialspace=True)


def uncomment(stream: TextIOWrapper):
    for line in stream:
        if line.startswith("#"):
            continue
        yield line



from typing import List

from pydantic import BaseModel

from hmmer_tables.csv_iter import csv_iter
from hmmer_tables.path_like import PathLike

__all__ = ["TBLScore", "TBLRow", "TBLIndex", "TBLDom", "read_tbl"]


class TBLIndex(BaseModel):
    name: str
    accession: str


class TBLScore(BaseModel):
    e_value: str
    score: str
    bias: str


class TBLDom(BaseModel):
    exp: str
    reg: int
    clu: int
    ov: int
    env: int
    dom: int
    rep: int
    inc: int


class TBLRow(BaseModel):
    target: TBLIndex
    query: TBLIndex
    full_sequence: TBLScore
    best_1_domain: TBLScore
    domain_numbers: TBLDom
    description: str


def read_tbl(filename: PathLike) -> List[TBLRow]:
    """
    Read tbl file type.

    Parameters
    ----------
    file
        File path or file stream.
    """
    rows = []
    with open(filename, "r") as file:
        for x in csv_iter(file):
            row = TBLRow(
                target=TBLIndex(name=x[0], accession=x[1]),
                query=TBLIndex(name=x[2], accession=x[3]),
                full_sequence=TBLScore(e_value=x[4], score=x[5], bias=x[6]),
                best_1_domain=TBLScore(e_value=x[7], score=x[8], bias=x[9]),
                domain_numbers=TBLDom(
                    exp=x[10],
                    reg=int(x[11]),
                    clu=int(x[12]),
                    ov=int(x[13]),
                    env=int(x[14]),
                    dom=int(x[15]),
                    rep=int(x[16]),
                    inc=int(x[17]),
                ),
                description=" ".join(x[18:]),
            )
            rows.append(row)
    return rows
